@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright © 2018 Damir Jelić <poljar@termina.org.uk>
+# Copyright © 2021 Famedly GmbH
 #
 # Permission to use, copy, modify, and/or distribute this software for
 # any purpose with or without fee is hereby granted, provided that the
@@ -33,7 +34,7 @@ from .events import (AccountDataEvent, FullyReadEvent, TagEvent,
                      RoomGuestAccessEvent, RoomHistoryVisibilityEvent,
                      RoomJoinRulesEvent, RoomMemberEvent, RoomNameEvent,
                      RoomTopicEvent, RoomAvatarEvent, TypingNoticeEvent,
-                     ReceiptEvent, Receipt)
+                     ReceiptEvent, Receipt, RoomUpgradeEvent)
 from .log import logger_group
 from .responses import RoomSummary, UnreadNotifications
 
@@ -78,6 +79,8 @@ class MatrixRoom:
         self.tags: Dict[str, Optional[Dict[str, float]]] = {}
         self.unread_notifications: int = 0
         self.unread_highlights: int = 0
+        self.members_synced: bool = False
+        self.replacement_room: Union[str, None] = None
         # yapf: enable
 
     @property
@@ -376,6 +379,9 @@ class MatrixRoom:
         elif isinstance(event, RoomEncryptionEvent):
             self.encrypted = True
 
+        elif isinstance(event, RoomUpgradeEvent):
+            self.replacement_room = event.replacement_room
+
         elif isinstance(event, PowerLevelsEvent):
             self.power_levels.update(event.power_levels)
 
@@ -435,23 +441,18 @@ class MatrixRoom:
         )
 
     @property
-    def members_synced(self) -> bool:
-        """Check if the room member state is fully synced.
-
-        Room members can be missing from the room if syncs are done using lazy
-        member loading, the room summary will contain the full member count but
-        other member info will be missing.
-
-        A `joined_members` request should be done for this room to populate the
-        member list. This is crucial for encrypted rooms before sending any
-        messages.
-        """
+    def joined_count(self) -> int:
         try:
-            _, joined, invited = self._summary_details()
+            return self._summary_details()[1]
         except ValueError:
-            return True
+            return len(tuple(u for u in self.users.values() if not u.invited))
 
-        return joined + invited == len(self.users)
+    @property
+    def invited_count(self) -> int:
+        try:
+            return self._summary_details()[2]
+        except ValueError:
+            return len(tuple(u for u in self.users.values() if u.invited))
 
     @property
     def member_count(self) -> int:
